(* ::Package:: *)

BeginPackage["beamlib`"]


PointLoad::usage="Concentrated point load P at x :{el,PointLoad,x,P}";
UniformLoad::usage="Uniform load q : {el,UniformLoad,0,q}";
LinearLoad::usage="A trapezoid load from x1 to x2 with values q1 to q2 :{el,LinearLoad,{x1,x2},{q1,q2}}";
MomentPointLoad::usage="Concentrated Moment M at point x : {el,MomentPointLoad,x,M}";
AxialPointLoad::usage="Concentrated axial point load N0 at x: {el,AxialPointLoad,x,N0}";
AxialLinearLoad::usage="A trapezoid axial load from x1 to x2 with values n1 to n2 :{el,AxialLinearLoad,{x1,x2},{n1,n2}}";

BeamNewStructure::usage="BeamInitStructure[struct], inits the struct for a new frame.";
BeamSetNodes::usage="BeamSetNodes[struct,nodes], sets the nodes of the structure. nodes is in the form: nodes={node1,node2...} with
each node in the form {x,y, constraintX, constraintY, constraintZ, Load/DispX,Load/DispY, Load/DispZ}. 
Examples:
Clamped node at point {0,0}: {0,0,1,1,1,0,0,0}
Simple supported node at point {2,3}: {2,3,1,1,0, 0,0,0}
Free node at point {3,4} with node load X=5,M=6 : {3,4,0,0,0, 5,0,6}
Clamped node at point {0,0} with prescribed displacementY=0.02 : {0,0,1,1,1,0,0.02,0}";
BeamSetElements::usage="BeamSetElements[struct,elements], sets the elements of the structure. elements in the form: {el1,el2,...} 
with each element in the form {node1, node2, E, I , A}. 
node1 is left node, node2 is right node, E is elastic modulus, I inertia, and A area of section";
BeamSetLoads::usage="BeamSetLoads[struct,loads], sets the loads of the structure. loads is in the form: {load1,load2,...} 
with each load in the form: {elementNumber,typeOfLoad,x,value}. 
Possibles types of Load are: PointLoad,UniformLoad,LinearLoad,MomentPointLoad,AxialPointLoad,AxialLinearLoad";
BeamSolve::usage="BeamSolve[struct], solves the structure and stores in the struct the results.
Options:
Assumptions : Assumption made to simplify symbolic results. Can be useful to include assumptions on values>0 and relations
between them.
Limit : List of limits for symbolic calculation of results. Can be useful to calculate symbolic results on extreme values of parameters";
BeamElementResults::usage="BeamElementResults[struct,options], shows the element results.
Options:
TitleColor, RowColors, Precision";
BeamNodeResults::usage="BeamNodeResults[struct,options], shows the node results.
Options:
TitleColor, RowColors, Precision";
BeamElementM::usage="BeamElementM[el,struct,prec,scale] returns the M table of element el in the form:
{{0,M0*scale},..,{xi+prec,Mi*scale},..,{len,M1*scale}}
with step over the length of element equal to prec, scaled with scale. The calculations are exact.
Scale has default value 1.0 and can be omitted.";
BeamElementQ::usage="BeamElementQ[el,struct,prec,scale] returns the Q table of element el in the form:
{{0,Q0*scale},..,{xi+prec,Qi*scale},..,{len,Q1*scale}}
with step over the length of element equal to prec, scaled with scale. The calculations are exact.
Scale has default value 1.0 and can be omitted.";
BeamElementN::usage="BeamElementN[el,struct,prec,scale] returns the N table of element el in the form:
{{0,N0*scale},..,{xi+prec,Ni*scale},..,{len,N1*scale}}
with step over the length of element equal to prec, scaled with scale. The calculations are exact.
Scale has default value 1.0 and can be omitted.";
BeamElementUt::usage="BeamElementUt[el,struct,prec,scale] returns the rotation table of element el in the form:
{{0,Ut0*scale},..,{xi+prec,Uti*scale},..,{len,Ut1*scale}} with step over the length of element equal to prec, scaled with scale. 
The calculation involves numerical integration, so the results are not exact. Scale has default value 1.0 and can be omitted.";
BeamElementUy::usage="BeamElementUy[el,struct,prec,scale] returns the Y deformation table of element el in the form:
{{0,Uy0*scale},..,{xi+prec,Uyi*scale},..,{len,Uy1*scale}} with step over the length of element equal to prec, scaled with scale. 
The calculation involves numerical integration, so the results are not exact. Scale has default value 1.0 and can be omitted.";
BeamElementUx::usage="BeamElementUx[el,struct,prec,scale] returns the X deformation table of element el in the form:
{{0,Ux0*scale},..,{xi+prec,Uxi*scale},..,{len,Ux1*scale}} with step over the length of element equal to prec, scaled with scale. 
The calculation involves numerical integration, so the results are not exact. Scale has default value 1.0 and can be omitted.";
BeamElementDeformation::usage="BeamElementDeformation[el,struct,prec,scale] returns the shape Uglobal deformation table of element el
in the form: {{0,scale{dUx0,dUy0},..,{xi+prec,scale{dUxi,dUyi}},..,{len,scale{dUx1,dUy1}}} with step over the length of element equal 
to prec, scaled with scale. 
The calculation involves numerical integration, so the results are not exact. Scale has default value 1.0 and can be omitted.";
BeamDrawDiagram::usage="BeamDrawDiagram[struct,options], draws several result diagrams. 
You have to run BeamSolve for the struct before running BeamDrawDiagram.
It's the only function for now, that cannot be used for symbolic results.
Options:
Color : The color of the diagram. Default color varies for each diagram type.
Diagram : The type of the diagram. Possible values are: \"Loads\",\"M\",\"Q\",\"N\",\"Ux\",\"Uy\",\"Ut\",\"Ug\".
Precision : The calculation precision step for the diagram. Default value is 0.01, wich divides the element in 1/100 pieces.
Scale : The scale of the diagram. Default value is 0.5 wich uses 50% of diagonal size of the graphics view divided by golden ratio,
as the maximum size of the diagram.
TextPrecision : The text precision for the numbers of the diagram. Default value: 0.01
Constraints : Draw constraints. Default value : True
NodeNumbers : Draw node numbers. Default value : True
ElementNumbers : Draw element numbers. Default value : True
Arrows : For each element draw a directional arrow from left to right element. Default value : True
TextStyle : List of tyle for the text of the diagram. Default value : {Larger,Bold}
Assumptions : Not used for now. Assumptions made so we can draw and symbolic results.
";


Begin["`Private`"]


(* calculations *)
(* a simple structure is composed as str1={Nodes,Elements,Loads,ElementProperties}; *)


PointLoad=1;UniformLoad=2;MomentPointLoad=3;AxialPointLoad=4;LinearLoad=5;AxialLinearLoad=6;
LeftEnd=1;RightEnd=2;BothEnds=0;


(* Nodes,Elements,Loads,ElementProperties,Options *)
BeamNewStructure[]:={{},{},{},{}};
BeamSetNodes[struct_,nodes_]:={nodes,struct[[2]],struct[[3]],struct[[4]]};
BeamSetElements[struct_,elements_]:={struct[[1]],elements,struct[[3]],struct[[4]]};
BeamSetLoads[struct_,loads_]:={struct[[1]],struct[[2]],loads,struct[[4]]};


NumOfNodes[struct_]:=Dimensions[struct[[1]]][[1]];
NumOfElements[struct_]:=Dimensions[struct[[2]]][[1]];
NumOfLoads[struct_]:=Dimensions[struct[[3]]][[1]];
(* NodeUsage returns the number of times each node is being used *)
NodeUsage[struct_]:=BinCounts[Flatten[(struct[[2]])[[All, {1, 2}]]], {1, NumOfNodes[struct] + 1, 1}];
(* NodeOutsideTest returns 1,0 for each node if the node is on the edge or inside *)
NodeOutsideTest[struct_]:= Map[If[# <= 1, 0, 1] &, NodeUsage[struct]];
(* NodeConnectivity returns if each node is connected or not *)
NodeConnectivity[struct_]:= Map[If[# >= 1, 1, 0] &, NodeOutsideTest[struct] + (struct[[1]])[[All, 5]]];
ElementConnectivity[struct_]:=Module[{c},
	c = Map[Part[NodeConnectivity[struct], #] &, (struct[[2]])[[All, {1, 2}]]];
	f[x_] := If[x[[1]] == 1 && x[[2]] == 1, BothEnds, If[x[[1]] == 1, LeftEnd, If[x[[2]] == 1, RightEnd]]];
	c= Map[f, c]; Clear[f];c];
(*Calculate Length,Angle and init zero FEL, FEG, QEL, QEG of elements \ *)
CalcElementProperties[struct_]:=Module[{t,elp},
	t = Map[Part[(struct[[1]])[[All, {1, 2}]], #] &,  (struct[[2]])[[All, {1, 2}]]];
	f[x_] := Module[{dx, dy, d1, s},
		dx=x[[2, 1]] - x[[1, 1]]; dy = x[[2, 2]] - x[[1, 2]];
		s=Sqrt[dx^2 + dy^2];
		d1=ArcTan[dx,dy];
		{s, d1, Table[0, {i, 6}], Table[0, {i, 6}], Table[0, {i, 6}]}];
	elp = Map[f, t]; Clear[f];
	{struct[[1]],struct[[2]],struct[[3]],elp}];
(* calc fixed end loads for Point load *)
PointLoadFixedEnd[x_, P_, s_, endType_] := Module[{},
   Switch[endType,
    LeftEnd,
    {0, -(x^3 - 3 x^2*s + 2*s^3)/(2*s^3)*
      P, +x*(s - x)*(1 + (s - x)/s)/(2*s)*P,
     0, -x^2 (x - 3 s)/(2*s^3)*P, 0},
    RightEnd,
    {0, -x^2 (x - 3 s)/(2*s^3)*P, 0,
     0, -(x^3 - 3 x^2*s + 2*s^3)/(2*s^3)*
      P, -x*(s - x)*(1 + (s - x)/s)/(2*s)*P},
    BothEnds,
    {0, -(x - s)^2*(2 x + s)/s^3*P, x (s - x)^2/s^2 *P,
     0, -(3 s - 2 x)*x^2/s^3*P, -(s - x) x^2 /s^2 *P}
    ]
   ];
(* calc fixed end loads for Uniform load *)
UniformLoadFixedEnd[q_, s_, endType_] := Module[{},
   Switch[endType,
    LeftEnd,
    {0, -5*q*s/8, q*s^2/8,
     0, -3*q*s/8, 0},
    RightEnd,
    {0, -3*q*s/8, 0,
     0, -5*q*s/8, -q*s^2/8},
    BothEnds,
    {0, -q*s/2, q*s^2/12,
     0, -q*s/2, -q*s^2/12}
    ]
   ];
(* calc fixed end loads for moment point load *)
MomentPointLoadFixedEnd[x_, M0_,s_, endType_] := Module[{},
   Switch[endType,
    LeftEnd,
    {0, -3M0 (2s-x)x/(2s^3), -M0(1+3x (x-2s)/(2s^2)),
     0, 3M0 (2s-x)x/(2s^3), 0},
    RightEnd,
    {0, 3M0 (2s-x)x/(2s^3), 0,
     0, -3M0 (2s-x)x/(2s^3), M0 (1+3x (x-2s)/(2s^2))},
    BothEnds,
    {0, -6M0 (s-x)x/s^3, -M0 (s-3x)(s-x)/s^2,
     0, 6M0 (s-x)x/s^3, M0 (2s-3x)x/s^2}
    ]
   ];
AxialPointLoadFixedEnd[x0_, P_,s_, endType_] := Module[{},
    {P (s-x0)/s, 0, 0,
     P*x0/s, 0, 0}
];
AxialLinearLoadFixedEnd[x_,P_,s_, endType_] := Module[{n1,n2,x1,x2},
   n1=P[[1]];n2=P[[2]];
   x1=x[[1]];x2=x[[2]];
	{(x2 - x1) (3s (n1+n2) -n1 (2x1+x2)- n2 (x1+2x2))/(6 s),0,0,
	(x2 - x1) (n1 (2x1+x2) + n2 (x1+2x2))/(6 s),0,0}
   ];

(* calc fixed end loads for Trapezoid Partial Load, p={pa,pb} , x={xa,xb} *)
LinearLoadFixedEnd[x_,q_,s_,endType_]:=Module[{pa,pb,x1,x2},
	pa=q[[1]];pb=q[[2]];
	x1=x[[1]];x2=x[[2]];
	Switch[endType,
	LeftEnd,
	{0,
	((x1-x2) (20 s^3 (pa+pb)+(4 pa+pb) x1^3+(3 pa+2 pb) x1^2 x2+(2 pa+3 pb) x1 x2^2+(pa+4 pb) x2^3+5 s (-(3 pa+pb) x1^2-2 (pa+pb) x1 x2-(pa+3 pb) x2^2)))/(40 s^3),
	-(1/(120 s^2))(x1-x2) (15 s (-(3 pa+pb) x1^2-2 (pa+pb) x1 x2-(pa+3 pb) x2^2)+3 ((4 pa+pb) x1^3+(3 pa+2 pb) x1^2 x2+(2 pa+3 pb) x1 x2^2+(pa+4 pb) x2^3)+20 s^2 (pa (2 x1+x2)+pb (x1+2 x2))),
	0,
	(x1^3 (5 s (3 pa+pb)-(4 pa+pb) x1)+(pa-pb) x1^2 (-5 s+x1) x2+(pa-pb) x1 (-5 s+x1) x2^2-(5 s (pa+3 pb)+(-pa+pb) x1) x2^3+(pa+4 pb) x2^4)/(40 s^3),
	0
	},
	RightEnd,
	{0,
	(x1^3 (5 s (3 pa+pb)-(4 pa+pb) x1)+(pa-pb) x1^2 (-5 s+x1) x2+(pa-pb) x1 (-5 s+x1) x2^2-(5 s (pa+3 pb)+(-pa+pb) x1) x2^3+(pa+4 pb) x2^4)/(40 s^3),
	0,
	0,
	-((x1-x2) (20 s^3 (pa+pb)+(4 pa+pb) x1^3+(3 pa+2 pb) x1^2 x2+(2 pa+3 pb) x1 x2^2+(pa+4 pb) x2^3+5 s (-(3 pa+pb) x1^2-2 (pa+pb) x1 x2-(pa+3 pb) x2^2)))/(40 s^3),
	(1/(120 s^2))(x1-x2) (15 s (-(3 pa+pb) x1^2-2 (pa+pb) x1 x2-(pa+3 pb) x2^2)+3 ((4 pa+pb) x1^3+(3 pa+2 pb) x1^2 x2+(2 pa+3 pb) x1 x2^2+(pa+4 pb) x2^3)+20 s^2 (pa (2 x1+x2)+pb (x1+2 x2)))
	},
	BothEnds,
	{0,
	(1/(20 s^3)) (x1-x2) (10 s^3 (pa+pb)+5 s (-(3 pa+pb) x1^2-2 (pa+pb) x1 x2-(pa+3 pb) x2^2)+2 ((4 pa+pb) x1^3+(3 pa+2 pb) x1^2 x2+(2 pa+3 pb) x1 x2^2+(pa+4 pb) x2^3)),
	-(1/(60 s^2))(x1-x2) (10 s (-(3 pa+pb) x1^2-2 (pa+pb) x1 x2-(pa+3 pb) x2^2)+3 ((4 pa+pb) x1^3+(3 pa+2 pb) x1^2 x2+(2 pa+3 pb) x1 x2^2+(pa+4 pb) x2^3)+10 s^2 (pa (2 x1+x2)+pb (x1+2 x2))),
	0,
	(5 s (x1-x2) ((3 pa+pb) x1^2+2 (pa+pb) x1 x2+(pa+3 pb) x2^2)+2 (-(4 pa+pb) x1^4+(pa-pb) x1^3 x2+(pa-pb) x1^2 x2^2+(pa-pb) x1 x2^3+(pa+4 pb) x2^4))/(20 s^3),
	(5 s (x1-x2) ((3 pa+pb) x1^2+2 (pa+pb) x1 x2+(pa+3 pb) x2^2)+3 (-(4 pa+pb) x1^4+(pa-pb) x1^3 x2+(pa-pb) x1^2 x2^2+(pa-pb) x1 x2^3+(pa+4 pb) x2^4))/(60 s^2)
	}]
];
FixedEndLoads[load_,elemProperties_,elemConnectivity_] := Module[{s, c},
   s = elemProperties[[load[[1]] ]][[1]];
   c = elemConnectivity[[load[[1]] ]];
   Switch[load[[2]],
    PointLoad, PointLoadFixedEnd[load[[3]], load[[4]], s, c ],
    UniformLoad, UniformLoadFixedEnd[load[[4]], s, c],
	MomentPointLoad,MomentPointLoadFixedEnd[load[[3]], load[[4]],s,c],
	LinearLoad,LinearLoadFixedEnd[load[[3]],load[[4]],s,c],
	AxialPointLoad,AxialPointLoadFixedEnd[load[[3]],load[[4]],s, c],
	AxialLinearLoad,AxialLinearLoadFixedEnd[load[[3]],load[[4]],s,c]
    ]
   ];


(* computes both local and global fixed end loads *)
CalcFixedEndLoads[struct_]:=Module[
{elp,elemConnectivity,loads,t,tt,w1,k,nloads,nelems},
elp=struct[[4]];(* element properties *)
elemConnectivity=ElementConnectivity[struct];
loads=struct[[3]]; (* element loads *)
nloads=NumOfLoads[struct];
t={};
For[k=1,k <= nloads,k++,
	tt=FixedEndLoads[loads[[k]],elp,elemConnectivity];
	(*Print[k,":",N[tt]];*)
	AppendTo[t,tt];
];
For[k=1,k <= nloads, k++,
	w1=loads[[k,1]]; (* element number *)
    elp[[w1,3]]=elp[[w1,3]]+t[[k]];
   ];
(* now elp[[i,3]] holds the local fixed end loads *)
nelems=NumOfElements[struct];
For[k = 1, k <= nelems, k++,
	t = elp[[k, 2]];(*angle*)
	elp[[k, 4, 1]] =Cos[t]*elp[[k, 3, 1]] -  Sin[t]*elp[[k, 3, 2]];
	elp[[k, 4, 2]] =Sin[t]*elp[[k, 3, 1]] +  Cos[t]*elp[[k, 3, 2]];
	elp[[k, 4, 3]] =elp[[k, 3, 3]];
	elp[[k, 4, 4]] =Cos[t]*elp[[k, 3, 4]] - Sin[t]*elp[[k, 3, 5]];
	elp[[k, 4, 5]] =Sin[t]*elp[[k, 3, 4]] + Cos[t]*elp[[k, 3, 5]];
    elp[[k, 4, 6]] =elp[[k, 3, 6]];
];
{struct[[1]],struct[[2]],struct[[3]],elp}
]


(* local stiffness matrix *)
Klocal[E_,I_,A_,L_] := 
   {
    {E A/L, 0, 0, -E A/L, 0, 0},
    {0, 12 E I/L^3, -6 E I/L^2, 0, -12 E I/L^3, -6 E I/L^2},
    {0, -6 E I /L^2, 4 E I/L, 0, 6 E I/L^2, 2 E I/L},
    {-E A/L, 0, 0, E A/L, 0, 0},
    {0, -12 E I/L^3, 6 E I/L^2, 0, 12 E I/L^3, 6 E I/L^2},
    {0, -6 E I /L^2, 2 E I/L, 0, 6 E I/L^2, 4 E I/L}
    };
Rotation[t_] := {
   {Cos[t], Sin[t], 0, 0, 0, 0},
   {-Sin[t], Cos[t], 0, 0, 0, 0},
   {0, 0, 1, 0, 0, 0},
   {0, 0, 0, Cos[t], Sin[t], 0},
   {0, 0, 0, -Sin[t], Cos[t], 0},
   {0, 0, 0, 0, 0, 1}
   };
Kglobal[t_,klocal_] :=  Transpose[Rotation[t]].klocal.Rotation[t];
LocalDOFs[el_,struct_] := Module[{i, j,elems},
   elems=struct[[2]];
   i = elems[[el, 1]];
   j = elems[[el, 2]];
   {(i - 1)*3 + 1, (i - 1)*3 + 2, (i - 1)*3 + 3,
    (j - 1)*3 + 1, (j - 1)*3 + 2, (j - 1)*3 + 3}
   ];


Options[BeamSolve]={Assumptions->{},Limit->{}};
BeamSolve[struct_,OptionsPattern[]]:=Module[
{NumNodes,NumElements,NumLoads,nodeUsage,nodeOutsideTest,nodeConnectivity,elemConnectivity,
str,nodes,elems,loads,elemProperties,assump,limit,
A,B,x0,k,w1,w2,i,j,r1,ug,ul,sign,l,t,g},

nodes=struct[[1]];elems=struct[[2]];loads=struct[[3]];elemProperties={};
assump=OptionValue[Assumptions];limit=OptionValue[Limit];
NumNodes=NumOfNodes[struct];
NumElements=NumOfElements[struct];
NumLoads=NumOfNodes[struct];
nodeUsage=NodeUsage[struct];
nodeOutsideTest=NodeOutsideTest[struct];
nodeConnectivity=NodeConnectivity[struct];
elemConnectivity=ElementConnectivity[struct];
str=CalcElementProperties[struct];
str=CalcFixedEndLoads[str];
elemProperties=str[[4]];
(* now assemble A,B matrices *)
A = Table[Table[0, {k, NumNodes*3}], {k, NumNodes*3}];
B = Table[0, {k, NumNodes*3}];
For[k = 1, k <= NumElements, k++,
   t = LocalDOFs[k,str];
   l = Klocal[elems[[k,3]],elems[[k,4]],elems[[k,5]],elemProperties[[k,1]]]; (* E,I,A,L *)
   r1 = elemProperties[[k,2]];(* angle *) 
   g = Kglobal[r1,l];
   For[i = 1, i <= 6, i++,
    w1 = t[[i]];
    For[j = 1, j <= 6, j++,
     w2 = t[[j]];
     A[[w1, w2]] = A[[w1, w2]] + g[[i, j]];
    ];
    B[[w1]] = B[[w1]] + elemProperties[[k, 4, i]];
  ];
];
(* apply nodal loads *)
For[k = 1, k <= NumNodes, k++,
    t = nodes[[k]];
    If[t[[3]] == 0 ,
     i = (k - 1)*3 + 1;
     B[[i]] = B[[i]] + t[[6]]];
    If[t[[4]] == 0 ,
     i = (k - 1)*3 + 2;
     B[[i]] = B[[i]] + t[[7]]];
    If[t[[5]] == 0 ,
     i = (k - 1)*3 + 3;
     B[[i]] = B[[i]] + t[[8]]];
];
(* add constraints *)
For[i = 1, i <= NumNodes*3, i++,
    k = Quotient[i - 1, 3] + 1;
    j = Mod[i - 1, 3];
    If[nodes[[k, 3 + j]] != 0,
     A[[i, All]] = Table[0, {NumNodes*3}];
     A[[i, i]] = 1;
     B[[i]] = nodes[[k, 6 + j]];
     ];
];
(* now solve *)
A=Simplify[A,Assumptions->assump];
B=Simplify[B,Assumptions->assump];
x0=Simplify[LinearSolve[A,B],Assumptions->assump];
(* compute element forces for each element *)
For[k = 1, k <= NumElements, k++,
   t = elemProperties[[k, 2]];
   ug = x0[[LocalDOFs[k,str]]];
   ul = {Cos[t]*ug[[1]] + Sin[t]*ug[[2]], -Sin[t]*ug[[1]] + 
      Cos[t]*ug[[2]], ug[[3]],
     Cos[t]*ug[[4]] + Sin[t]*ug[[5]], -Sin[t]*ug[[4]] + 
      Cos[t]*ug[[5]], ug[[6]]};
   r1 = (Klocal[elems[[k,3]],elems[[k,4]],elems[[k,5]],elemProperties[[k,1]]]).ul;
   sign = {-1, 1, 1, 1, -1, -1};
   r1 = sign*(r1 - elemProperties[[k, 3]]);
   AppendTo[elemProperties[[k]], r1];
];
elemProperties=Simplify[elemProperties,Assumptions->assump]; (* update element properties *)
For[k=1, k<=NumElements,k++,
	For[i=1,i<=6,i++, (*6 values *)
		For[j=1,j<=Length[limit],j++ ,(* for each limit rule *)
			t=Limit[elemProperties[[k,6,i]],limit[[j]] ,Assumptions->assump];
			elemProperties[[k,6,i]]=t;
		];
	];
];
str[[4]]=elemProperties;
(*compute 3 reactions and 3 global displacements for each node*)
For[k = 1, k <= NumNodes, k++,
   AppendTo[nodes[[k]], {0, 0, 0, 0, 0, 0}]
];
For[k = 1, k <= NumElements, k++,
    t = elemProperties[[k, 2]];
    i = elems[[k, 1]]; j = elems[[k, 2]];
    r1 = elemProperties[[k, 6]];(* element forces *)
    If[nodes[[i, 3]] != 0,   nodes[[i, 9, 1]] = nodes[[i, 9, 1]] - Cos[t]*r1[[1]] - Sin[t]*r1[[2]]];
    If[nodes[[i, 4]] != 0,   nodes[[i, 9, 2]] = nodes[[i, 9, 2]] + Sin[t]*r1[[1]] - Cos[t]*r1[[2]]];
    If[nodes[[i, 5]] != 0,   nodes[[i, 9, 3]] = nodes[[i, 9, 3]] - r1[[3]]];
    If[nodes[[j, 3]] != 0,   nodes[[j, 9, 1]] = nodes[[j, 9, 1]] - Cos[t]*r1[[4]] - Sin[t]*r1[[5]]];
    If[nodes[[j, 4]] != 0,   nodes[[j, 9, 2]] = nodes[[j, 9, 2]] + Sin[t]*r1[[4]] - Cos[t]*r1[[5]]];
    If[nodes[[j, 5]] != 0,   nodes[[j, 9, 3]] = nodes[[j, 9, 3]] - r1[[6]]];
    nodes[[i, 9, 4]] = x0[[(i - 1)*3 + 1]];
    nodes[[i, 9, 5]] = x0[[(i - 1)*3 + 2]];
    nodes[[i, 9, 6]] = x0[[(i - 1)*3 + 3]];
    nodes[[j, 9, 4]] = x0[[(j - 1)*3 + 1]];
    nodes[[j, 9, 5]] = x0[[(j - 1)*3 + 2]];
    nodes[[j, 9, 6]] = x0[[(j - 1)*3 + 3]];
];
nodes=Simplify[nodes,Assumptions->assump]; (* update node properties *)
For[k=1,k<=NumNodes,k++,
	For[i=1,i<=6,i++, (* 6 values *)
		For[j=1,j<=Length[limit],j++, (* for each limit rule *)
				t=Limit[nodes[[k,9,i]],limit[[j]] ,Assumptions->assump];
				nodes[[k,9,i]]=t;
			];
		];
	];
str[[1]]=nodes;
str];


Options[BeamElementResults]={TitleColor->Lighter[Orange],RowColors->{White,LightGray},Precision->0.001};
BeamElementResults[struct_,OptionsPattern[]]:=Module[
{tab1,grid1,k,r1,numElements,elp,pre},
tab1 = {{"Element", "Na", "Qa", "Ma", "Nb", "Qb", "Mb"}};
numElements=NumOfElements[struct];
elp=struct[[4]];  
pre=OptionValue[Precision];
For[k = 1, k <= numElements, k++,
    r1 = elp[[k, 6]];
    If[NumberQ[r1[[1]]],r1[[1]]=Round[r1[[1]],pre]];
    If[NumberQ[r1[[2]]],r1[[2]]=Round[r1[[2]],pre]];
	If[NumberQ[r1[[3]]],r1[[3]]=Round[r1[[3]],pre]];
	If[NumberQ[r1[[4]]],r1[[4]]=Round[r1[[4]],pre]];
	If[NumberQ[r1[[5]]],r1[[5]]=Round[r1[[5]],pre]];
	If[NumberQ[r1[[6]]],r1[[6]]=Round[r1[[6]],pre]];
    AppendTo[ tab1, {k, r1[[1]], r1[[2]], r1[[3]], r1[[4]], r1[[5]],  r1[[6]]}];
  ];
grid1=Grid[tab1, Frame -> True, Dividers -> {{{2 -> True}}, 2 -> True}, 
 Background -> {None, {{{OptionValue[RowColors][[1]],OptionValue[RowColors][[2]]}}, 
    1 -> OptionValue[TitleColor]}}];
grid1
];


Options[BeamNodeResults]={TitleColor->Lighter[Orange],RowColors->{White,LightGray},Precision->0.001};
BeamNodeResults[struct_,OptionsPattern[]]:=Module[
{tab1,grid1,k,r1,numNodes,nodes,pre},
tab1 = {{"Node", "Rx", "Ry", "Mz", "Ux", "Uy", "Rz"}};
numNodes=NumOfNodes[struct];
nodes=struct[[1]];
pre=OptionValue[Precision];
For[k = 1, k <= numNodes, k++,
  r1 = nodes[[k, 9]];
  If[NumberQ[r1[[1]]],r1[[1]]=Round[r1[[1]],pre]];
  If[NumberQ[r1[[2]]],r1[[2]]=Round[r1[[2]],pre]];
  If[NumberQ[r1[[3]]],r1[[3]]=Round[r1[[3]],pre]];
  If[NumberQ[r1[[4]]],r1[[4]]=Round[r1[[4]],pre]];
  If[NumberQ[r1[[5]]],r1[[5]]=Round[r1[[5]],pre]];
  If[NumberQ[r1[[6]]],r1[[6]]=Round[r1[[6]],pre]];
   AppendTo[  tab1, {k, r1[[1]], r1[[2]], r1[[3]], r1[[4]], r1[[5]],  r1[[6]]}];
];
grid1=Grid[tab1, Frame -> True, Dividers -> {{{2 -> True}}, 2 -> True}, 
 Background -> {None, {{{OptionValue[RowColors][[1]],OptionValue[RowColors][[2]]}}, 
    1 -> OptionValue[TitleColor]}}];
grid1
];


BeamElementUx[el_,struct_,prec_,scale_:1.0]:= Module[{k,kk,r1,r2,ul,ug,ug2,ul2,u,elp,angle,elem,EE,AA,mm,area,x1,x2,v1,v2,xx,L,t},
	elp=struct[[4]]; (* element properties *)
	elem=struct[[2,el]]; (* element *)
	L=elp[[el,1]];  (* length *)
	angle=elp[[el,2]]; (* angle *)
	EE=elem[[3]]; (*Elastic modulus *)
	AA=elem[[5]]; (* area of section *)
	r1=(struct[[1]])[[elem[[1]],9]]; (* [[1]] left node, at [[9]] the results *)
	r2=(struct[[1]])[[elem[[2]],9]]; (* [[1]] left node, at [[9]] the results *)
	ug=r1[[4;;6]]; ug2=r2[[4;;6]];(* global displacements *)
	ul = {Cos[angle]*ug[[1]]+Sin[angle]*ug[[2]], -Sin[angle]*ug[[1]]+Cos[angle]*ug[[2]], ug[[3]]}; (* local displ *)
	ul2 = {Cos[angle]*ug2[[1]]+Sin[angle]*ug2[[2]], -Sin[angle]*ug2[[1]]+Cos[angle]*ug2[[2]], ug2[[3]]}; 
	u=ul[[1]]; (* x deformation at left node *)
	mm=BeamElementN[el,struct,prec,1.0]; (* N axial force in element *)
	t={{0,scale*u}};
	(*integrate axial diagram to get X deformation *)
	For[k=1,k<=Length[mm]-1,k++,
		x1=mm[[k,1]];x2=mm[[k+1,1]];
		v1=mm[[k,2]];v2=mm[[k+1,2]];
		area=(v1+v2)*(x2-x1)/2 / (EE AA); 
		u=u+area;
		AppendTo[t,{x2,scale*u}];
	];
	(* now trapezoid error correction from Ux of right node *)
    kk=Length[t];
	For[k=1,k<=kk,k++,
		t[[k,2]]=t[[k,2]]-(t[[kk,2]]-scale*ul2[[1]])*k/kk;
	];
	AppendTo[t,{L,scale*ul2[[1]]}]; (* append right node *)
t
]


BeamElementUy[el_,struct_,prec_,scale_:1.0]:= Module[{k,kk,r1,r2,ul,ug,ug2,ul2,u,elp,angle,elem,EE,II,mm,area,x1,x2,v1,v2,L,t},
	elp=struct[[4]]; (* element properties *)
	elem=struct[[2,el]]; (*element *)
	L=elp[[el,1]];  (* length *)
	angle=elp[[el,2]]; (* angle *)
	EE=elem[[3]]; (*Elastic modulus *)
	II=elem[[4]]; (* Inertia *)
	r1=(struct[[1]])[[elem[[1]],9]]; (* [[1]] left node, at [[9]] the results *)
	r2=(struct[[1]])[[elem[[2]],9]]; (* [[1]] left node, at [[9]] the results *)
	ug=r1[[4;;6]]; ug2=r2[[4;;6]];(* global displacements *)
	ul = {Cos[angle]*ug[[1]]+Sin[angle]*ug[[2]], -Sin[angle]*ug[[1]]+Cos[angle]*ug[[2]], ug[[3]]}; (* local displ *)
	ul2 = {Cos[angle]*ug2[[1]]+Sin[angle]*ug2[[2]], -Sin[angle]*ug2[[1]]+Cos[angle]*ug2[[2]], ug2[[3]]}; 
	u=ul[[2]]; (* y deformation at left node *)
	mm=BeamElementUt[el,struct,prec,1.0];
	t={{0,scale*u}};
	(*integrate rotation to get deformation *)
	For[k=1,k<=Length[mm]-1,k++,
		x1=mm[[k,1]];x2=mm[[k+1,1]];
		v1=mm[[k,2]];v2=mm[[k+1,2]];
		area=(v1+v2)*(x2-x1)/2; 
		u=u-area;
		AppendTo[t,{x2,scale*u}];
	];
	(* now trapezoid error correction from Ut of right node *)
    kk=Length[t];
	For[k=1,k<=kk,k++,
		t[[k,2]]=t[[k,2]]-(t[[kk,2]]-scale*ul2[[2]])*k/kk;
	];
	
t
]


(* returns rotation of element*)
BeamElementUt[el_,struct_,prec_,scale_:1.0]:= Module[{k,kk,r1,r2,u,elp,elem,EE,II,area,x1,x2,v1,v2,L,t,mt},
	elp=struct[[4]]; (* element properties *)
	elem=struct[[2,el]]; (* element *)
	L=elp[[el,1]];  (* length *)
	EE=elem[[3]]; (*Elastic modulus *)
	II=elem[[4]]; (* Inertia *)
	r1=(struct[[1]])[[elem[[1]],9]]; (* [[1]] left node, at [[9]] the results *)
	r2=(struct[[1]])[[elem[[2]],9]]; (* [[1]] left node, at [[9]] the results *)
	u=r1[[6]];
	mt=BeamElementM[el,struct,prec,1.0]; (* gets M diagram *)
	t={{0,scale*u}};
    (* integration of M/EI to get rotation *)
	For[k=1,k<=Length[mt]-1,k++,
		x1=mt[[k,1]];x2=mt[[k+1,1]];
		v1=mt[[k,2]];v2=mt[[k+1,2]];
		area=(v1+v2)*(x2-x1)/2; 
		u=u-area/(EE II);
		AppendTo[t,{x2,scale*u}];
	];
	(* now trapezoid error correction from Ut of right node *)
    kk=Length[t];
	For[k=1,k<=kk,k++,
		t[[k,2]]=t[[k,2]]-(t[[kk,2]]-scale*r2[[6]])*k/kk;
	];
	AppendTo[t,{L,scale*r2[[6]]}]; (* append right node *)
t
]


BeamElementDeformation[el_,struct_,prec_,scale_:1.0]:=Module[{elp,elem,nodes,len,angle,p1,p2,p,pp,k,xx,x1,y1,r1},
	elp=struct[[4]];
	elem=struct[[2,el]]; (* element *)
	nodes={ struct[[1,elem[[1]],1;;2]],struct[[1,elem[[2]] ,1;;2]]}; (* line of element *)
	elp=struct[[4,el]]; (* element properties *)
	len=elp[[1]]; (* length  *)
	angle=elp[[2]]; (* angle  *)
	p1=BeamElementUx[el,struct,prec,1.0]; 
	p2=BeamElementUy[el,struct,prec,1.0]; 
	p={};
	For[k=1,k<=Length[p1],k++,
		xx=p1[[k,1]]; (* x over the length of the element *)
		x1=(*Cos[angle]xx +*)Cos[angle] scale p1[[k,2]]- scale Sin[angle] p2[[k,2]]; (* p2[[k,1]] is the same *)
		y1=(*Sin[angle]xx *)Sin[angle] scale p1[[k,2]]+ scale Cos[angle] p2[[k,2]];
		AppendTo[p,{xx,{x1,y1}}]; 
	];
p
];


BeamElementM[el_,struct_,prec_,scale_:1.0]:=Module[{elp,L,t,x,k,r1,r2,m,loads,elemLoads,loadType,a,b,pa,pb,x1,x2},
	elp=struct[[4]]; (* element properties *)
	loads=struct[[3]]; (* loads *)
	L=elp[[el,1]]; (* length of element *)
	r1=elp[[el,6]]; (* element local forces *)
	elemLoads={}; (* loads in the current element *)
	t={};
	For[k=1,k<=NumOfLoads[struct],k++, If[loads[[k,1]]==el,AppendTo[elemLoads,loads[[k]] ]] ]; 
	For[x=0,x<L,x=x+prec,
		m=r1[[3]]+r1[[2]]*x;
		For[k=1,k<=Length[elemLoads],k++,
			loadType=elemLoads[[k,2]];
			a=elemLoads[[k,3]]; b=elemLoads[[k,4]];
			Switch[loadType,
             PointLoad, If[x > a, m = m - (x - a)*b],
             UniformLoad, m = m - b*x^2 /2,
			 MomentPointLoad,If[x>a, m = m - b],
			 AxialPointLoad,m=m,
			 AxialLinearLoad,m=m,
	         LinearLoad,x1=a[[1]];x2=a[[2]];pa=b[[1]];pb=b[[2]];
			              If[x<x1,m=m,If[x<=x2, m=m- (((x - x1)^2 (pb (-x + x1) + pa (x + 2 x1 - 3 x2)))/(6 (x1 - x2))),
							If[x>x2, m=m - (1/6 (x1 - x2) (pa (-3 x + 2 x1 + x2) + pb (-3 x + x1 + 2 x2)))]]];
		     ](* end switch *)
		]; (*end for k *)
	AppendTo[t,{x,scale*m}];
	];
	AppendTo[t,{L,scale*r1[[6]]}]; (* append right node *)
t
];


BeamElementQ[el_,struct_,prec_,scale_:1.0]:=Module[{elp,L,t,x,k,r1,r2,q,loads,elemLoads,loadType,a,b,pa,pb,x1,x2},
	elp=struct[[4]]; (* element properties *)
	loads=struct[[3]]; (* loads *)
	L=elp[[el,1]]; (* length of element *)
	r1=elp[[el,6]]; (* element local forces *)
	elemLoads={}; (* loads in the current element *)
	t={};
	For[k=1,k<=NumOfLoads[struct],k++, If[loads[[k,1]]==el,AppendTo[elemLoads,loads[[k]] ]] ]; 
	For[x=0,x<L,x=x+prec,
		q=r1[[2]];
		For[k=1,k<=Length[elemLoads],k++,
			loadType=elemLoads[[k,2]];
			a=elemLoads[[k,3]]; b=elemLoads[[k,4]];
			Switch[loadType,
              PointLoad, If[x > a, q = q - b],
              UniformLoad, q = q - b*x,
			  MomentPointLoad,q = q,
			  AxialPointLoad,q=q,
			  AxialLinearLoad,q=q,
	          LinearLoad,x1=a[[1]];x2=a[[2]];pa=b[[1]];pb=b[[2]];
				            If[x<x1,q=q,  If[x<=x2,q=q- (pa x - pa x1 + ((pa - pb) (x - x1)^2)/(2 (x1 - x2))), 
							q=q- (x2-x1)(pa+pb)/2 ]]
		];(* end switch *)
		]; (*end for k *)
	AppendTo[t,{x,scale*q}];
	];
	AppendTo[t,{L,scale*r1[[5]]}]; (*append right node *)
t
];


BeamElementN[el_,struct_,prec_,scale_:1.0]:=Module[{elp,L,t,x,k,r1,r2,n,loads,elemLoads,loadType,a,b,pa,pb,x1,x2},
	elp=struct[[4]]; (* element properties *)
	loads=struct[[3]]; (* loads *)
	L=elp[[el,1]]; (* length of element *)
	r1=elp[[el,6]]; (* element local forces *)
	elemLoads={}; (* loads in the current element *)
	t={};
	For[k=1,k<=NumOfLoads[struct],k++, If[loads[[k,1]]==el,AppendTo[elemLoads,loads[[k]] ]] ]; 
	For[x=0,x<L,x=x+prec,
		n=r1[[1]];
		For[k=1,k<=Length[elemLoads],k++,
			loadType=elemLoads[[k,2]];
			a=elemLoads[[k,3]]; b=elemLoads[[k,4]];
			Switch[loadType,
              PointLoad, n=n,
              UniformLoad, n=n,
			  MomentPointLoad, n=n,
			  AxialPointLoad, If[x>=a,n=n-b],
			  AxialLinearLoad, If[x>a[[2]],n=n - (b[[2]]+b[[1]])/2 (a[[2]]-a[[1]])];
							   If[x>=a[[1]] && x<=a[[2]],n=n - b[[1]](x-a[[1]]) - (b[[2]]-b[[1]])(x-a[[1]])^2/(2(a[[2]]-a[[1]])) ];,
	          LinearLoad, n=n
		];(* end switch *)
		]; (*end for k *)
	AppendTo[t,{x,scale*n}];
	];
	AppendTo[t,{L,scale*r1[[4]]}]; (*append right node*)
t
];


DrawConstraints[el_,struct_,scale_]:=Module[{n1,c1,gr,s11,s12,s21,s22,t1,t2},
s11={{1, -0.5}, {1, +0.5}}; s12={{-1, -0.5}, {-1, +0.5}} ;
s21={{-0.5, 1}, {+0.5, 1}}; s22={{-0.5, -1}, {+0.5, -1}} ;
t1=ScalingTransform[{scale,scale}];s11=t1[s11];s12=t1[s12];s21=t1[s21];s22=t1[s22];
n1=struct[[1,struct[[2,el,1]] ]];
t2=TranslationTransform[{n1[[1]],n1[[2]]}];
gr={};
c1=If[n1[[3]]!=0,{Line[t2[s11]],Line[t2[s12]]},Line[{}] ];AppendTo[gr,c1];
c1=If[n1[[4]]!=0,{Line[t2[s21]],Line[t2[s22]]},Line[{}] ];AppendTo[gr,c1];
c1=If[n1[[5]]!=0,Circle[{n1[[1]],n1[[2]]},scale*0.5],Line[{}] ];AppendTo[gr,c1];
n1=struct[[1,struct[[2,el,2]] ]];
t2=TranslationTransform[{n1[[1]],n1[[2]]}];
c1=If[n1[[3]]!=0,{Line[t2[s11]],Line[t2[s12]]},Line[{}] ];AppendTo[gr,c1];
c1=If[n1[[4]]!=0,{Line[t2[s21]],Line[t2[s22]]},Line[{}] ];AppendTo[gr,c1];
c1=If[n1[[5]]!=0,Circle[{n1[[1]],n1[[2]]},scale*0.5],Line[{}] ];AppendTo[gr,c1];
gr
];


(* returns list with 6 items in the form {{xmin,valuemin},{xmax,valuemax},size} for M,Q,N,Ux,Uy,Ut*)
FindMinMax[el_,struct_,precision_]:=Module[
{len,xmin,xmax,min,max,size,v,r1,tt,k},
len=struct[[4]][[el,1]]; (* length of element *)
r1={};
tt={BeamElementM[el,struct,precision,1],BeamElementQ[el,struct,precision,1],BeamElementN[el,struct,precision,1],
	BeamElementUx[el,struct,precision,1],BeamElementUy[el,struct,precision,1],BeamElementUt[el,struct,precision,1]};
For[k=1,k<=6,k++,
	v=tt[[k]];
	min=Min[ v[[All,2]] ]; xmin=v[[ (Position[v[[All,2]],min][[1,1]]),1]];
	max=Max[ v[[All,2]] ]; xmax=v[[ (Position[v[[All,2]],max][[1,1]]),1]];
	size=Max[Abs[{max-min,max,min}]];
	AppendTo[r1, {{xmin,min},{xmax,max},size} ];
];
r1
];


(* returns  1 minmaxwin={diagonal,{{xmin,ymin},{xmax,ymax}},
			2-8 sizeM,sizeQ,sizeN,sizeUx,sizeUy,sizeUt,sizeLoad,
			9,mel={ for each element the FindMixMax structure 
			*)
GetLimits[struct_,precision_]:=Module[
{nodes,numElements,numLoads,elp,box,sizeM,sizeQ,sizeN,sizeUx,sizeUy,sizeUt,sizeLoad, diagSize,i,j,k,m,q,n,len,mel,xp,loads,loadType},
	nodes=struct[[1]];
	numElements=NumOfElements[struct];
	numLoads=NumOfLoads[struct];
    elp=struct[[4]]; (* element properties *)
	loads=struct[[3]];
	box={{Min[nodes[[All,1]]],Min[nodes[[All,2]]]},{Max[nodes[[All,1]]],Max[nodes[[All,2]]]}};
	diagSize=Sqrt[ (box[[2,2]]-box[[1,2]])^2+(box[[2,1]]-box[[1,1]])^2];
	mel={}; (*min max size for for each element *)
    For[k=1, k<=numElements, k++, m=FindMinMax[k,struct,precision];AppendTo[mel,m]; ];
	sizeM=Max[mel[[All,1,3]]];sizeQ=Max[mel[[All,2,3]]];sizeN=Max[mel[[All,3,3]]];
	sizeUx=Max[mel[[All,4,3]]];sizeUy=Max[mel[[All,5,3]]];sizeUt=Max[mel[[All,6,3]]];
	sizeLoad=Max[Abs[{Max[loads[[All,4]]]-Min[loads[[All,4]]],Max[loads[[All,4]]],Min[loads[[All,4]]],
				Max[nodes[[All,6;;8]]]-Min[nodes[[All,6;;8]]],Max[nodes[[All,6;;8]]],Min[nodes[[All,6;;8]]]}]];
	
{{diagSize,box},sizeM,sizeQ,sizeN,sizeUx,sizeUy,sizeUt,sizeLoad,mel}
];



DrawElement[el_,struct_,drawNodeNumbers_,drawElementNumbers_,drawArrow_,drawConstraints_,diagSize_,textStyle_]:=Module[{g,p,elem,nodes,s1,s2},
elem=struct[[2,el]];
nodes={struct[[1,elem[[1]],1;;2]],struct[[1,elem[[2]],1;;2]]};
s1=textStyle;PrependTo[s1,Blue];
s2=textStyle;PrependTo[s2,Red];AppendTo[s2,Underlined];
g={Black,AbsoluteThickness[2.0],Line[nodes],AbsolutePointSize[3.0]};
If[drawNodeNumbers==True,
	p={
	Style[Text[elem[[1]],nodes[[1]],{-1.2,+1.2}],s1],s1[[1]],Point[nodes[[1]] ],
	Style[Text[elem[[2]],nodes[[2]],{-1.2,+1.2}],s1],s1[[1]],Point[nodes[[2]] ]
	};
	AppendTo[g,p];
];
If[drawElementNumbers==True,
	p={Style[Text[el,(nodes[[1]]+nodes[[2]])/2,{-1.2,+1.2}],s2]
	};
	AppendTo[g,p];
];
If[drawArrow==True,
	p={Arrow[{nodes[[1]],(nodes[[1]]+nodes[[2]])/2}]};
	AppendTo[g,p];
];
If[drawConstraints==True,
	p=DrawConstraints[el,struct,diagSize*0.02];
	AppendTo[g,p];	
];
g
];


Options[DrawElementDeformation]={Precision->0.01,Scale->0.5,Constraints->True,TextPrecision->0.01,
NodeNumbers->True,ElementNumbers->True,Arrows->True,TextStyle->{Larger,Bold}};
DrawElementDeformation[struct_,el_,minmax_,color_,OptionsPattern[]]:=Module[
{nodes,elp,elem,g,len,angle,sc1,k,p,p1,pp,t1,t2,xx,x1,y1,ge,tprec},
	(*minmax={{diagSize,box},sizeM,sizeQ,sizeN,sizeUx,sizeUy,sizeUt,sizeLoad,mel}*)
	tprec=OptionValue[TextPrecision];
	elem=struct[[2,el]]; (* element *)
	nodes={ struct[[1,elem[[1]],1;;2]],struct[[1,elem[[2]] ,1;;2]]}; (* line of element *)
	elp=struct[[4,el]]; (* element properties *)
	len=elp[[1]]; (* length  *)
	angle=elp[[2]]; (* angle  *)
	sc1=OptionValue[Scale] minmax[[1,1]]/Max[minmax[[5]],minmax[[6]]]/GoldenRatio; (* diagSize/Max(sizeUx,sizeUy)*)
	ge=DrawElement[el,struct,OptionValue[NodeNumbers],
					OptionValue[ElementNumbers],OptionValue[Arrows],OptionValue[Constraints],minmax[[1,1]],OptionValue[TextStyle] ];
	p=BeamElementDeformation[el,struct,OptionValue[Precision],sc1];
	p1=p[[All,1]]; p=p[[All,2]]; (*p1 is x, p is {dx,dy} *)
	pp={};
	For[k=1,k<=Length[p1],k++,
		xx=p1[[k]]; (* x over the length of the element *)
		x1=nodes[[1,1]]+Cos[angle]xx +p[[k,1]];
		y1=nodes[[1,2]]+Sin[angle]xx +p[[k,2]];
		AppendTo[pp,{x1,y1}];
	];
	t1=Round[struct[[1,elem[[1]],9,4;;5]],tprec];
	t2=Round[struct[[1,elem[[2]],9,4;;5]],tprec];
	g=Graphics[{
	ge,
	AbsoluteThickness[2.5],color,Dashed,Line[pp],
	Black,Opacity[1.0],Red,Point[pp[[1]]],
	color,AbsoluteThickness[1.0],Dotted,Line[{pp[[1]],pp[[-1]]}],
	Black,Dashing[{}],
	Style[Text[t1,pp[[1]],{0,Sign[t1[[2]]]}],OptionValue[TextStyle]],
	Style[Text[t2,pp[[-1]],{0,Sign[t2[[2]]]}],OptionValue[TextStyle]]
	},{}];
g
];


IsAwayQ[value_,limit1_,limit2_,prec_]:=Module[{q},
	q=If[(Abs[value-limit1]<=prec) || (Abs[value-limit2]<=prec),False,True];
q
];


Options[DrawElementUx]={Precision->0.01,Scale->0.5,Constraints->True,TextPrecision->0.01,
NodeNumbers->True,ElementNumbers->True,Arrows->True,TextStyle->{Larger,Bold}};
DrawElementUx[struct_,el_,minmax_,color_,OptionsPattern[]]:=Module[
{nodes,elp,elem,g,len,angle,sc1,p,r3,t1,t2,pt1,pt2,m1,p1,p2,drawp1,drawp2,tprec,ul,ul2,ug,ug2,ge},
	(*minmax={{diagSize,box},sizeM,sizeQ,sizeN,sizeUx,sizeUy,sizeUt,sizeLoad,mel}*)
    tprec=OptionValue[TextPrecision];
	elem=struct[[2,el]]; (* element *)
	nodes={ struct[[1,elem[[1]],1;;2]],struct[[1,elem[[2]] ,1;;2]]}; (* line of element *)
	elp=struct[[4,el]]; (*  element properties *)
	len=elp[[1]]; (* length  *)
	angle=elp[[2]]; (* angle  *)
	sc1=OptionValue[Scale] minmax[[1,1]]/minmax[[5]]/GoldenRatio; (* diagSize/sizeUx*)
	ge=DrawElement[el,struct,OptionValue[NodeNumbers],
					OptionValue[ElementNumbers],OptionValue[Arrows],OptionValue[Constraints],minmax[[1,1]],OptionValue[TextStyle] ];
	p=BeamElementUx[el,struct,OptionValue[Precision],sc1]; 
	m1=minmax[[9,el,4]]; (* {xmin,valuemin},{xmax,valuemax}} sizeUx *)
	p1={{m1[[2,1]],0},{m1[[2,1]],sc1*m1[[2,2]]}}; (*max line *)
	drawp1=IsAwayQ[p1[[1,1]],0,len,1.5*OptionValue[Precision]]; (* check if falls on the edge *)	
	p2={{m1[[1,1]],0},{m1[[1,1]],sc1*m1[[1,2]]}}; (*min line *)
	drawp2=IsAwayQ[p2[[1,1]],0,len,1.5*OptionValue[Precision]];
	PrependTo[p,{0,0}];AppendTo[p,{len,0}];
	r3=RotationTransform[angle];p=r3[p];p1=r3[p1];p2=r3[p2];
	r3=TranslationTransform[{nodes[[1,1]],nodes[[1,2]]}];p=r3[p];p1=r3[p1];p2=r3[p2];
	ug=struct[[1,elem[[1]],9,4;;6]]; ug2=struct[[1,elem[[2]],9,4;;6]];(* global displacements *)
	ul = {Cos[angle]*ug[[1]]+Sin[angle]*ug[[2]], -Sin[angle]*ug[[1]]+Cos[angle]*ug[[2]], ug[[3]]}; (* local displacements *)
	ul2 = {Cos[angle]*ug2[[1]]+Sin[angle]*ug2[[2]], -Sin[angle]*ug2[[1]]+Cos[angle]*ug2[[2]], ug2[[3]]}; 
	t1=ul[[1]]; t1=Round[t1,tprec];
	t2=ul2[[1]]; t2=Round[t2,tprec];
	pt1={m1[[2,1]],Round[m1[[2,2]],tprec]};
	pt2={m1[[1,1]],Round[m1[[1,2]],tprec]};
	g=Graphics[{
	ge,AbsoluteThickness[1.0],color,Line[p],Opacity[0.3],Polygon[p],
	Black,Opacity[1.0],
	Style[Text[t1,p[[2]],{0,Sign[t1]}] , OptionValue[TextStyle]],
	Style[Text[t2,p[[-2]],{0,Sign[t2]} ],OptionValue[TextStyle]],
	If[drawp1,Style[Text[pt1,p1[[2]],{0,1}],OptionValue[TextStyle]],{}],
	If[drawp2,Style[Text[pt2,p2[[2]],{0,-1}],OptionValue[TextStyle]],{}],
	Dashed,If[drawp1,Line[p1],{}],If[drawp2,Line[p2],{}]
	},{}];
g
];


Options[DrawElementUy]={Precision->0.01,Scale->0.5,Constraints->True,TextPrecision->0.01,
NodeNumbers->True,ElementNumbers->True,Arrows->True,TextStyle->{Larger,Bold}};
DrawElementUy[struct_,el_,minmax_,color_,OptionsPattern[]]:=Module[
{nodes,elp,elem,g,len,angle,sc1,p,r3,t1,t2,pt1,pt2,m1,p1,p2,drawp1,drawp2,tprec,ul,ul2,ug,ug2,ge},
	(*minmax={{diagSize,box},sizeM,sizeQ,sizeN,sizeUx,sizeUy,sizeUt,sizeLoad,mel}*)
	tprec=OptionValue[TextPrecision];
	elem=struct[[2,el]]; (*  element *)
	nodes={ struct[[1,elem[[1]],1;;2]],struct[[1,elem[[2]] ,1;;2]]}; (* line of element *)
	elp=struct[[4,el]]; (*  element properties *)
	len=elp[[1]]; (* length  *)
	angle=elp[[2]]; (* angle  *)
	sc1=OptionValue[Scale] minmax[[1,1]]/minmax[[6]]/GoldenRatio; (* diagSize/sizeUy*)
	ge=DrawElement[el,struct,OptionValue[NodeNumbers],
					OptionValue[ElementNumbers],OptionValue[Arrows],OptionValue[Constraints],minmax[[1,1]],OptionValue[TextStyle] ];
	p=BeamElementUy[el,struct,OptionValue[Precision],sc1]; 
	m1=minmax[[9,el,5]]; (* {xmin,valuemin},{xmax,valuemax}} sizeUy *)
	p1={{m1[[2,1]],0},{m1[[2,1]],sc1*m1[[2,2]]}}; (*max line *)
	drawp1=IsAwayQ[p1[[1,1]],0,len,1.5*OptionValue[Precision]];(* check if falls on the edge *)	
	p2={{m1[[1,1]],0},{m1[[1,1]],sc1*m1[[1,2]]}}; (*min line *)
	drawp2=IsAwayQ[p2[[1,1]],0,len,1.5*OptionValue[Precision]]; (* check if falls on the edge *)	
	PrependTo[p,{0,0}];AppendTo[p,{len,0}];
	r3=RotationTransform[angle];p=r3[p];p1=r3[p1];p2=r3[p2];
	r3=TranslationTransform[{nodes[[1,1]],nodes[[1,2]]}];p=r3[p];p1=r3[p1];p2=r3[p2];
	ug=struct[[1,elem[[1]],9,4;;6]]; ug2=struct[[1,elem[[2]],9,4;;6]];(* global displacements *)
	ul = {Cos[angle]*ug[[1]]+Sin[angle]*ug[[2]], -Sin[angle]*ug[[1]]+Cos[angle]*ug[[2]], ug[[3]]}; (* local displacements *)
	ul2 = {Cos[angle]*ug2[[1]]+Sin[angle]*ug2[[2]], -Sin[angle]*ug2[[1]]+Cos[angle]*ug2[[2]], ug2[[3]]}; 
	t1=ul[[2]]; t1=Round[t1,tprec];
	t2=ul2[[2]]; t2=Round[t2,tprec];
	pt1={m1[[2,1]],Round[m1[[2,2]],tprec]};
	pt2={m1[[1,1]],Round[m1[[1,2]],tprec]};
	g=Graphics[{
	ge,AbsoluteThickness[1.0],color,Line[p],Opacity[0.3],Polygon[p],
	Black,Opacity[1.0],
	Style[Text[t1,p[[2]],{0,Sign[t1]}] , OptionValue[TextStyle]],
	Style[Text[t2,p[[-2]],{0,Sign[t2]} ],OptionValue[TextStyle]],
	If[drawp1,Style[Text[pt1,p1[[2]],{0,1}],OptionValue[TextStyle]],{}],
	If[drawp2,Style[Text[pt2,p2[[2]],{0,-1}],OptionValue[TextStyle]],{}],
	Dashed,If[drawp1,Line[p1],{}],If[drawp2,Line[p2],{}]
	},{}];
g
];


Options[DrawElementUt]={Precision->0.01,Scale->0.5,Constraints->True,TextPrecision->0.01,
NodeNumbers->True,ElementNumbers->True,Arrows->True,TextStyle->{Larger,Bold}};
DrawElementUt[struct_,el_,minmax_,color_,OptionsPattern[]]:=Module[
{nodes,elp,elem,g,len,angle,sc1,p,r3,t1,t2,pt1,pt2,m1,p1,p2,drawp1,drawp2,tprec,ge},
	(*minmax={{diagSize,box},sizeM,sizeQ,sizeN,sizeUx,sizeUy,sizeUt,sizeLoad,mel}*)
	tprec=OptionValue[TextPrecision];
	elem=struct[[2,el]]; (* element *)
	nodes={ struct[[1,elem[[1]],1;;2]],struct[[1,elem[[2]] ,1;;2]]}; (* line of element *)
	elp=struct[[4,el]]; (*  element properties *)
	len=elp[[1]]; (* length  *)
	angle=elp[[2]]; (* angle *)
	sc1=OptionValue[Scale] minmax[[1,1]]/minmax[[7]]/GoldenRatio; (* diagsize/sizeUt*)
	ge=DrawElement[el,struct,OptionValue[NodeNumbers],
					OptionValue[ElementNumbers],OptionValue[Arrows],OptionValue[Constraints],minmax[[1,1]],OptionValue[TextStyle] ];
	p=BeamElementUt[el,struct,OptionValue[Precision],sc1]; (* plot with Precision step *)
	m1=minmax[[9,el,6]]; (* {xmin,valuemin},{xmax,valuemax}} sizeUt*)
	p1={{m1[[2,1]],0},{m1[[2,1]],sc1*m1[[2,2]]}}; (*max line *)
	drawp1=IsAwayQ[p1[[1,1]],0,len,1.5*OptionValue[Precision]]; (* check if falls on the edge *)	
	p2={{m1[[1,1]],0},{m1[[1,1]],sc1*m1[[1,2]]}}; (*min line *)
	drawp2=IsAwayQ[p2[[1,1]],0,len,1.5*OptionValue[Precision]];(* check if falls on the edge *)	
	PrependTo[p,{0,0}];AppendTo[p,{len,0}];
	r3=RotationTransform[angle];p=r3[p];p1=r3[p1];p2=r3[p2];
	r3=TranslationTransform[{nodes[[1,1]],nodes[[1,2]]}];p=r3[p];p1=r3[p1];p2=r3[p2];
	t1=struct[[1,elem[[1]],9,6]]; t1=Round[t1,tprec];
	t2=struct[[1,elem[[2]],9,6]]; t2=Round[t2,tprec];
	pt1={m1[[2,1]],Round[m1[[2,2]],tprec]};
	pt2={m1[[1,1]],Round[m1[[1,2]],tprec]};
	g=Graphics[{
	ge,AbsoluteThickness[1.0],color,Line[p],Opacity[0.3],Polygon[p],
	Black,Opacity[1.0],
	Style[Text[t1,p[[2]],{0,Sign[t1]}] , OptionValue[TextStyle]],
	Style[Text[t2,p[[-2]],{0,Sign[t2]} ], OptionValue[TextStyle]],
	If[drawp1,Style[Text[pt1,p1[[2]],{0,1}], OptionValue[TextStyle]],{}],
	If[drawp2,Style[Text[pt2,p2[[2]],{0,-1}], OptionValue[TextStyle]],{}],
	Dashed,If[drawp1,Line[p1],{}],If[drawp2,Line[p2],{}]
	},{}];
g
];


Options[DrawElementM]={Precision->0.01,Scale->0.5,Constraints->True,TextPrecision->0.01,
NodeNumbers->True,ElementNumbers->True,Arrows->True,TextStyle->{Larger,Bold}};
DrawElementM[struct_,el_,minmax_,color_,OptionsPattern[]]:=Module[
{nodes,elp,elem,g,len,angle,sc1,p,r3,t1,t2,pt1,pt2,m1,p1,p2,drawp1,drawp2,ge,tprec},
	(*minmax={{diagSize,box},sizeM,sizeQ,sizeN,sizeUx,sizeUy,sizeUt,sizeLoad,mel}*)
	tprec=OptionValue[TextPrecision];
	elem=struct[[2,el]]; (* element element *)
	nodes={ struct[[1,elem[[1]],1;;2]],struct[[1,elem[[2]] ,1;;2]]}; (* line of element *)
	elp=struct[[4,el]]; (* element properties *)
	len=elp[[1]]; (* length of element *)
	angle=elp[[2]]; (* angle of element *)
	sc1=OptionValue[Scale] minmax[[1,1]]/minmax[[2]]/GoldenRatio; (* diagSize/sizeM*)
	ge=DrawElement[el,struct,OptionValue[NodeNumbers],
					OptionValue[ElementNumbers],OptionValue[Arrows],OptionValue[Constraints],minmax[[1,1]],OptionValue[TextStyle] ];
	p=BeamElementM[el,struct,OptionValue[Precision],-sc1];
	m1=minmax[[9,el,1]]; (* {xmin,valuemin},{xmax,valuemax},sizeM *)
	p1={{m1[[2,1]],0},{m1[[2,1]],-sc1*m1[[2,2]]}}; (*max line *)
	drawp1=IsAwayQ[p1[[1,1]],0,len,1.5*OptionValue[Precision]]; (* check if falls on the edge *)	
	p2={{m1[[1,1]],0},{m1[[1,1]],-sc1*m1[[1,2]]}}; (*min line *)
	drawp2=IsAwayQ[p2[[1,1]],0,len,1.5*OptionValue[Precision]]; (* check if falls on the edge *)	
	PrependTo[p,{0,0}];AppendTo[p,{len,0}];
	r3=RotationTransform[angle];p=r3[p];p1=r3[p1];p2=r3[p2];
	r3=TranslationTransform[{nodes[[1,1]],nodes[[1,2]]}];p=r3[p];p1=r3[p1];p2=r3[p2];
	t1=Round[elp[[6,3]],tprec]; t2=Round[elp[[6,6]],tprec]; (*text at left and right *)
	pt1={m1[[2,1]],Round[m1[[2,2]],tprec]}; pt2={m1[[1,1]],Round[m1[[1,2]],tprec]}; (* text at max min *)
	g=Graphics[{
	ge,AbsoluteThickness[1.0],color,Line[p],Opacity[0.3],Polygon[p],
	Black,Opacity[1.0],
	Style[Text[t1,p[[2]],{0,Sign[t1]}] , OptionValue[TextStyle]],
	Style[Text[t2,p[[-2]],{0,Sign[t2]} ],OptionValue[TextStyle]],
	If[drawp1,Style[Text[pt1,p1[[2]],{0,1}],OptionValue[TextStyle]],{}],
	If[drawp2,Style[Text[pt2,p2[[2]],{0,-1}],OptionValue[TextStyle]],{}],
	Dashed,If[drawp1,Line[p1],{}],If[drawp2,Line[p2],{}]
	},{}];
g
];


Options[DrawElementQ]={Precision->0.01,Scale->0.5,Constraints->True,TextPrecision->0.01,
NodeNumbers->True,ElementNumbers->True,Arrows->True,TextStyle->{Larger,Bold}};
DrawElementQ[struct_,el_,minmax_,color_,OptionsPattern[]]:=Module[
{nodes,elp,elem,g,len,angle,sc1,p,r3,t1,t2,pt1,pt2,m1,p1,p2,drawp1,drawp2,tprec,ge},
	(*minmax={{diagSize,box},sizeM,sizeQ,sizeN,sizeUx,sizeUy,sizeUt,sizeLoad,mel}*)
	tprec=OptionValue[TextPrecision];
	elem=struct[[2,el]]; (* element *)
	nodes={ struct[[1,elem[[1]],1;;2]],struct[[1,elem[[2]] ,1;;2]]}; (* line of element *)
	elp=struct[[4,el]]; (* element properties *)
	len=elp[[1]]; (* length  *)
	angle=elp[[2]]; (* angle*)
	sc1=OptionValue[Scale] minmax[[1,1]]/minmax[[3]]/GoldenRatio; (* diagSize/sizeQ *)
    ge=DrawElement[el,struct,OptionValue[NodeNumbers],
					OptionValue[ElementNumbers],OptionValue[Arrows],OptionValue[Constraints],minmax[[1,1]],OptionValue[TextStyle] ];
	p=BeamElementQ[el,struct,OptionValue[Precision],sc1];
	m1=minmax[[9,el,2]];  (* {{xmin,valuemin},{xmax,valuemax}} ,sizeQ*)
	p1={{m1[[2,1]],0},{m1[[2,1]],sc1*m1[[2,2]]}}; (*max line *)
	drawp1=IsAwayQ[p1[[1,1]],0,len,1.5*OptionValue[Precision]]; (* check if falls on the edge *)	
	p2={{m1[[1,1]],0},{m1[[1,1]],sc1*m1[[1,2]]}}; (*min line *)
	drawp2=IsAwayQ[p2[[1,1]],0,len,1.5*OptionValue[Precision]]; (* check if falls on the edge *)	
	PrependTo[p,{0,0}];AppendTo[p,{len,0}];
	r3=RotationTransform[angle];p=r3[p];p1=r3[p1];p2=r3[p2];
	r3=TranslationTransform[{nodes[[1,1]],nodes[[1,2]]}];p=r3[p];p1=r3[p1];p2=r3[p2];
	t1=Round[elp[[6,2]],tprec]; (* left text *)
	t2=Round[elp[[6,5]],tprec]; (* right text *)
	pt1={m1[[2,1]],Round[m1[[2,2]],tprec]}; (*max text *)
	pt2={m1[[1,1]],Round[m1[[1,2]],tprec]}; (*min text *)
	g=Graphics[{
	ge,AbsoluteThickness[1.0],color,Line[p],Opacity[0.3],Polygon[p],
	Black,Opacity[1.0],
	Style[Text[t1,p[[2]],{0,-Sign[t1]} ] , OptionValue[TextStyle]],
	Style[Text[t2,p[[-2]],{0,-Sign[t2]} ],OptionValue[TextStyle]],
	If[drawp1,Style[Text[pt1,p1[[2]],{0,-1}],OptionValue[TextStyle]],{}],
	If[drawp2,Style[Text[pt2,p2[[2]],{0,1}],OptionValue[TextStyle]],{}],
	Dashed,If[drawp1,Line[p1],{}],If[drawp2,Line[p2],{}]
	},{}];
g
];


Options[DrawElementN]={Precision->0.01,Scale->0.5,Constraints->True,TextPrecision->0.01,
NodeNumbers->True,ElementNumbers->True,Arrows->True,TextStyle->{Larger,Bold}};
DrawElementN[struct_,el_,minmax_,color_,OptionsPattern[]]:=Module[
{nodes,elp,elem,g,len,angle,sc1,p,r3,t1,t2,pt1,pt2,m1,p1,p2,drawp1,drawp2,tprec,ge},
	(*minmax={{diagSize,box},sizeM,sizeQ,sizeN,sizeUx,sizeUy,sizeUt,sizeLoad,mel}*)
	tprec=OptionValue[TextPrecision];
	elem=struct[[2,el]]; (* element *)
	nodes={ struct[[1,elem[[1]],1;;2]],struct[[1,elem[[2]] ,1;;2]]}; (* line of element *)
	elp=struct[[4,el]]; (* element properties *)
	len=elp[[1]]; (* length *)
	angle=elp[[2]]; (* angle *)
	sc1=OptionValue[Scale] minmax[[1,1]]/minmax[[4]]/GoldenRatio; (* diagSize/sizeN  *)
    ge=DrawElement[el,struct,OptionValue[NodeNumbers],
					OptionValue[ElementNumbers],OptionValue[Arrows],OptionValue[Constraints],minmax[[1,1]],OptionValue[TextStyle] ];
	p=BeamElementN[el,struct,OptionValue[Precision],sc1];
	m1=minmax[[9,el,3]]; (* {{xmin,valuemin},{xmax,valuemax}} ,sizeN*)
	p1={{m1[[2,1]],0},{m1[[2,1]],sc1*m1[[2,2]]}}; (*max line *)
	drawp1=IsAwayQ[p1[[1,1]],0,len,1.5*OptionValue[Precision]]; (* check if falls on the edge *)	
	p2={{m1[[1,1]],0},{m1[[1,1]],sc1*m1[[1,2]]}}; (*min line *)
	drawp2=IsAwayQ[p2[[1,1]],0,len,1.5*OptionValue[Precision]];  (* check if falls on the edge *)	
	PrependTo[p,{0,0}];AppendTo[p,{len,0}];
	r3=RotationTransform[angle];p=r3[p];p1=r3[p1];p2=r3[p2];
	r3=TranslationTransform[{nodes[[1,1]],nodes[[1,2]]}];p=r3[p];p1=r3[p1];p2=r3[p2];
	t1=Round[elp[[6,1]],tprec];
	t2=Round[elp[[6,4]],tprec];
	pt1={m1[[2,1]],Round[m1[[2,2]],tprec]};
	pt2={m1[[1,1]],Round[m1[[1,2]],tprec]};
	g=Graphics[{
	ge,AbsoluteThickness[1.0],color,Line[p],Opacity[0.3],Polygon[p],
	Black,Opacity[1.0],
	Style[Text[t1,p[[2]],{0,-Sign[t1]} ] , OptionValue[TextStyle]],
	Style[Text[t2,p[[-2]],{0,-Sign[t2]} ],OptionValue[TextStyle]],
	If[drawp1,Style[Text[pt1,p1[[2]],{0,-1}],OptionValue[TextStyle]],{}],
	If[drawp2,Style[Text[pt2,p2[[2]],{0,1}],OptionValue[TextStyle]],{}],
	Dashed,If[drawp1,Line[p1],{}],If[drawp2,Line[p2],{}]
	},{}];
g
];


DrawNodeLoads[n_,scale_,tprec_,style_]:=Module[{g,pp,pp1,r},
	g={};
	If[n[[3]]==0 && n[[6]]!=0,
		pp={n[[1;;2]],n[[1;;2]]+{scale*n[[6]],0}};
		AppendTo[g,{    Arrow[pp],
						Style[Text[Round[n[[6]],tprec],pp[[2]],{-1,0}],style]
					}];
	];
	If[n[[4]]==0 && n[[7]]!=0,
		pp={n[[1;;2]],n[[1;;2]]+{0,scale*n[[7]]}};
		AppendTo[g,{    Arrow[pp],
						Style[Text[Round[n[[7]],tprec],pp[[2]],{-1,0}],style]
					}];
	];
	If[n[[5]]==0 && n[[8]]!=0,
		r=scale*Abs[n[[8]]];
		pp={n[[1;;2]],n[[1;;2]]+{0,r}};
		pp1={{n[[1]]+Sin[Pi/8]*r,n[[2]]-Cos[Pi/8]*r},{n[[1]],n[[2]]-r}};
		AppendTo[g,{    Circle[pp[[1]],r,{-Pi/2,Pi}],Arrow[pp1],
						Style[Text[Round[n[[8]],tprec],pp[[2]],{0,-1}],style]
					}];
	];
g
];


Options[DrawElementLoads]={Precision->0.01,Scale->0.5,Constraints->True,TextPrecision->0.01,
NodeNumbers->True,ElementNumbers->True,Arrows->True,TextStyle->{Larger,Bold}};
DrawElementLoads[struct_,el_,minmax_,color_,OptionsPattern[]]:=Module[
{nodes,k,elp,loads,loadType,elem,g,len,angle,sc1,p,r3,r4,m1,pp,pp1,pp2,pp3,tprec,ge,n},
	(*minmax={{diagSize,box},sizeM,sizeQ,sizeN,sizeUx,sizeUy,sizeUt,sizeLoad,mel}*)
	tprec=OptionValue[TextPrecision];
	elem=struct[[2,el]]; (*  element *)
	nodes={ struct[[1,elem[[1]],1;;2]],struct[[1,elem[[2]] ,1;;2]]}; (* line of element *)
	elp=struct[[4,el]]; (* element properties *)
	len=elp[[1]]; (* length  *)
	angle=elp[[2]]; (* angle  *)
	loads=struct[[3]]; 
	sc1=OptionValue[Scale] minmax[[1,1]]/minmax[[8]]/GoldenRatio; (* diagSize/sizeLoad*)
	ge=DrawElement[el,struct,OptionValue[NodeNumbers],
					OptionValue[ElementNumbers],OptionValue[Arrows],OptionValue[Constraints],minmax[[1,1]],OptionValue[TextStyle] ];
	r3=RotationTransform[angle];
	r4=TranslationTransform[{nodes[[1,1]],nodes[[1,2]]}];
	p={AbsoluteThickness[1.0]};
	n=struct[[1,elem[[1]]]];AppendTo[p,DrawNodeLoads[n,sc1,tprec,OptionValue[TextStyle]]];
	n=struct[[1,elem[[2]]]];AppendTo[p,DrawNodeLoads[n,sc1,tprec,OptionValue[TextStyle]]];
	For[k=1,k<=NumOfLoads[struct],k++,
		loadType=loads[[k,2]];
		If[loads[[k,1]]==el,
			Switch[loadType,
				PointLoad,
					pp={{loads[[k,3]],sc1*loads[[k,4]]},{loads[[k,3]],0}};
					pp=r4[r3[pp]];
					AppendTo[p,{	Arrow[pp],Point[pp[[1]]],
									Style[Text[Round[Abs[loads[[k,4]]],tprec],pp[[1]],{0,-1}],OptionValue[TextStyle]]
								}];,
				AxialPointLoad,
					pp={{loads[[k,3]],minmax[[1,1]]*0.02},{loads[[k,3]]+sc1*loads[[k,4]],minmax[[1,1]]*0.02}};
					pp=r4[r3[pp]];
					AppendTo[p,{	Arrow[pp],Point[pp[[1]]],
									Style[Text[Round[Abs[loads[[k,4]]],tprec],pp[[2]],{0,-1}],OptionValue[TextStyle]]
								}];,
				UniformLoad,
					pp={{0,sc1*loads[[k,4]]},{len,sc1*loads[[k,4]]},{len,0},{0,0},{0,sc1*loads[[k,4]]}};pp=r4[r3[pp]]; 
					pp1={{len/2,sc1*loads[[k,4]]},{len/2,0}};pp1=r4[r3[pp1]];
					pp2={{len/4,sc1*loads[[k,4]]},{len/4,0}};pp2=r4[r3[pp2]];
					pp3={{3len/4,sc1*loads[[k,4]]},{3len/4,0}};pp3=r4[r3[pp3]];
					AppendTo[p,{	Line[pp],Opacity[0.3],Polygon[pp],Opacity[1.0],
									Arrow[pp1],Arrow[pp2],Arrow[pp3],
									Style[Text[Round[Abs[loads[[k,4]]],tprec],pp1[[1]],{0,-1}],OptionValue[TextStyle]]	
								}];,
				MomentPointLoad,
					pp1={loads[[k,3]],0}; pp1=r4[r3[pp1]];
					m1=Abs[sc1*loads[[k,4]]];
					pp2={{loads[[k,3]]-Cos[Pi/8]*m1,Sin[Pi/8]*m1},{loads[[k,3]]-m1,0}};pp2=r4[r3[pp2]];
					pp3={loads[[k,3]],m1}; pp3=r4[r3[pp3]];
					AppendTo[p,{Circle[pp1,m1,{-Pi/2,Pi}],Arrow[pp2],
								Style[Text[Round[loads[[k,4]],tprec],pp3,{0,-1}],OptionValue[TextStyle]]
								}];,
				LinearLoad,
					pp={{loads[[k,3,1]],sc1*loads[[k,4,1]]},{loads[[k,3,2]],sc1*loads[[k,4,2]]},
								{loads[[k,3,2]],0},{loads[[k,3,1]],0},{loads[[k,3,1]],sc1*loads[[k,4,1]]}};pp=r4[r3[pp]];
					pp1={{loads[[k,3,1]],sc1*loads[[k,4,1]]},{loads[[k,3,1]],0}};pp1=r4[r3[pp1]];
					pp3={{loads[[k,3,2]],sc1*loads[[k,4,2]]},{loads[[k,3,2]],0}};pp3=r4[r3[pp3]];
					pp2={{(loads[[k,3,1]]+loads[[k,3,2]])/2,sc1 (loads[[k,4,1]]+loads[[k,4,2]])/2},{(loads[[k,3,1]]+loads[[k,3,2]])/2,0}};
					pp2=r4[r3[pp2]];
					AppendTo[p,{    Line[pp],Opacity[0.3],Polygon[pp],Opacity[1.0],
									Arrow[pp1],Arrow[pp2],Arrow[pp3],
									Style[Text[Round[Abs[loads[[k,4,1]]],tprec],pp1[[1]],{0,-1}],OptionValue[TextStyle]],
									Style[Text[Round[Abs[loads[[k,4,2]]],tprec],pp3[[1]],{0,-1}],OptionValue[TextStyle]]
								}];,
				AxialLinearLoad,
					pp={{loads[[k,3,1]],sc1*loads[[k,4,1]]},{loads[[k,3,2]],sc1*loads[[k,4,2]]},
								{loads[[k,3,2]],0},{loads[[k,3,1]],0},{loads[[k,3,1]],sc1*loads[[k,4,1]]}};pp=r4[r3[pp]];
					pp1={{loads[[k,3,1]],sc1*loads[[k,4,1]]},{loads[[k,3,1]]+sc1*loads[[k,4,1]],sc1*loads[[k,4,1]]}};pp1=r4[r3[pp1]];
					pp3={{loads[[k,3,2]],sc1*loads[[k,4,2]]},{loads[[k,3,2]]+sc1*loads[[k,4,2]],sc1*loads[[k,4,2]]}};pp3=r4[r3[pp3]];
					pp2={{(loads[[k,3,1]]+loads[[k,3,2]])/2,sc1 (loads[[k,4,1]]+loads[[k,4,2]])/2},
						{(loads[[k,3,1]]+loads[[k,3,2]])/2+sc1 (loads[[k,4,1]]+loads[[k,4,2]])/2,sc1 (loads[[k,4,1]]+loads[[k,4,2]])/2}};
					pp2=r4[r3[pp2]];
					AppendTo[p,{    Line[pp],Opacity[0.3],Polygon[pp],Opacity[1.0],
									Arrow[pp1],Arrow[pp2],Arrow[pp3],
									Style[Text[Round[Abs[loads[[k,4,1]]],tprec],pp1[[1]],{0,-1}],OptionValue[TextStyle]],
									Style[Text[Round[Abs[loads[[k,4,2]]],tprec],pp3[[1]],{0,-1}],OptionValue[TextStyle]]
								}];
			];
		];
	];
	g=Graphics[{
	ge,color,p
	},{}];
g
];


P


names={ {"M","Moments","Moment"},
		{"Q","Shear","Vy"},
		{"N","Axial","Vx"},
		{"Loads","Load","q"},
		{"Ut","Rotation","Rot"},
		{"Uy","DY","dy"},
		{"Ux","DX","dx"},
		{"Ug","Deformation","Shape"}
};
GetDiagramName[s_]:=Module[{i,j,r},
	r=names[[1,1]];
	For[i=1,i<=8,i++,
		For[j=1,j<=3,j++,
			If[names[[i,j]]==s,r=names[[i,1]]];
		];
	];
r
];


Options[BeamDrawDiagram]={Color->None,Diagram->"M",TextPrecision->0.01,Precision->0.01,Scale->0.5,Constraints->True,
Assumptions->{},NodeNumbers->True,ElementNumbers->True,Arrows->True,TextStyle->{Larger,Bold}};
BeamDrawDiagram[struct_,OptionsPattern[]]:=Module[
{minmax,k,g,col,ass1,d,gt},
(*minmax={{diagSize,box},sizeM,sizeQ,sizeN,sizeUx,sizeUy,sizeUt,sizeLoad,mel}*)
col=OptionValue[Color];
ass1=OptionValue[Assumptions];
minmax=GetLimits[struct,OptionValue[Precision]];
g={};
d=GetDiagramName[OptionValue[Diagram]];
Switch[d,
"M",
If[col==None,col=Red];
For[k=1,k<=NumOfElements[struct],k++,
	AppendTo[g,DrawElementM[struct,k,minmax,col,Precision->OptionValue[Precision],Arrows->OptionValue[Arrows],
			Constraints->OptionValue[Constraints],Scale->OptionValue[Scale],TextPrecision->OptionValue[TextPrecision],
			NodeNumbers->OptionValue[NodeNumbers],ElementNumbers->OptionValue[ElementNumbers],TextStyle->OptionValue[TextStyle]]];
];,
"Q",
If[col==None,col=Blue];
For[k=1,k<=NumOfElements[struct],k++,
	AppendTo[g,DrawElementQ[struct,k,minmax,col,Precision->OptionValue[Precision],Arrows->OptionValue[Arrows],
			Constraints->OptionValue[Constraints],Scale->OptionValue[Scale],TextPrecision->OptionValue[TextPrecision],
			NodeNumbers->OptionValue[NodeNumbers],ElementNumbers->OptionValue[ElementNumbers],TextStyle->OptionValue[TextStyle]]];
];,
"N",
If[col==None,col=Darker[Green]];
For[k=1,k<=NumOfElements[struct],k++,
	AppendTo[g,DrawElementN[struct,k,minmax,col,Precision->OptionValue[Precision],Arrows->OptionValue[Arrows],
			Constraints->OptionValue[Constraints],Scale->OptionValue[Scale],TextPrecision->OptionValue[TextPrecision],
			NodeNumbers->OptionValue[NodeNumbers],ElementNumbers->OptionValue[ElementNumbers],TextStyle->OptionValue[TextStyle]]];
];,
"Loads",
If[col==None,col=Lighter[Black]];
For[k=1,k<=NumOfElements[struct],k++,
	AppendTo[g,DrawElementLoads[struct,k,minmax,col,Precision->OptionValue[Precision],Arrows->OptionValue[Arrows],
			Constraints->OptionValue[Constraints],Scale->OptionValue[Scale],TextPrecision->OptionValue[TextPrecision],
			NodeNumbers->OptionValue[NodeNumbers],ElementNumbers->OptionValue[ElementNumbers],TextStyle->OptionValue[TextStyle]]];
];,
"Ut",
If[col==None,col=Purple];
For[k=1,k<=NumOfElements[struct],k++,
	AppendTo[g,DrawElementUt[struct,k,minmax,col,Precision->OptionValue[Precision],Arrows->OptionValue[Arrows],
			Constraints->OptionValue[Constraints],Scale->OptionValue[Scale],TextPrecision->OptionValue[TextPrecision],
			NodeNumbers->OptionValue[NodeNumbers],ElementNumbers->OptionValue[ElementNumbers],TextStyle->OptionValue[TextStyle]]];
];,
"Uy",
If[col==None,col=Darker[Orange]];
For[k=1,k<=NumOfElements[struct],k++,
	AppendTo[g,DrawElementUy[struct,k,minmax,col,Precision->OptionValue[Precision],Arrows->OptionValue[Arrows],
			Constraints->OptionValue[Constraints],Scale->OptionValue[Scale],TextPrecision->OptionValue[TextPrecision],
			NodeNumbers->OptionValue[NodeNumbers],ElementNumbers->OptionValue[ElementNumbers],TextStyle->OptionValue[TextStyle]]];
];,
"Ux",
If[col==None,col=Darker[Orange]];
For[k=1,k<=NumOfElements[struct],k++,
	AppendTo[g,DrawElementUx[struct,k,minmax,col,Precision->OptionValue[Precision],Arrows->OptionValue[Arrows],
			Constraints->OptionValue[Constraints],Scale->OptionValue[Scale],TextPrecision->OptionValue[TextPrecision],
			NodeNumbers->OptionValue[NodeNumbers],ElementNumbers->OptionValue[ElementNumbers],TextStyle->OptionValue[TextStyle]]];
];,
"Ug",
If[col==None,col=Lighter[Black]];
For[k=1,k<=NumOfElements[struct],k++,
	AppendTo[g,DrawElementDeformation[struct,k,minmax,col,Precision->OptionValue[Precision],Arrows->OptionValue[Arrows],
			Constraints->OptionValue[Constraints],Scale->OptionValue[Scale],TextPrecision->OptionValue[TextPrecision],
			NodeNumbers->OptionValue[NodeNumbers],ElementNumbers->OptionValue[ElementNumbers],TextStyle->OptionValue[TextStyle]]];
];

]; (* end switch*)
Show[g]
];


End[]


EndPackage[]
